@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vteglyph.c,v 1.14 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -29,25 +28,26 @@
 #include "vtefc.h"
 #include "vteglyph.h"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(PACKAGE, String)
-#else
-#define _(String) String
-#define bindtextdomain(package,dir)
-#endif
+#include <glib/gi18n-lib.h>
 
 #define FONT_INDEX_FUDGE 10
 #define CHAR_WIDTH_FUDGE 10
-#define INVALID_GLYPH    -1
+#define INVALID_GLYPH    GINT_TO_POINTER(-1)
 
 static FT_Face _vte_glyph_cache_face_for_char(struct _vte_glyph_cache *cache,
 					      gunichar c);
 
-static int
-_vte_direct_compare(gconstpointer a, gconstpointer b)
+void
+_vte_glyph_free(struct _vte_glyph *glyph)
 {
-	return GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b);
+	g_free(glyph);
+}
+
+static void
+_vte_cached_glyph_free(gpointer glyph)
+{
+	if (glyph != INVALID_GLYPH)
+		_vte_glyph_free(glyph);
 }
 
 struct _vte_glyph_cache *
@@ -56,11 +56,11 @@ _vte_glyph_cache_new(void)
 	struct _vte_glyph_cache *ret;
 	int error;
 
-	ret = g_malloc(sizeof(struct _vte_glyph_cache));
+	ret = g_slice_new(struct _vte_glyph_cache);
 
-	ret->patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
+	ret->patterns = g_ptr_array_new();
 	ret->faces = NULL;
-	ret->cache = g_tree_new(_vte_direct_compare);
+	ret->cache = g_hash_table_new_full(NULL, NULL, NULL, _vte_cached_glyph_free);
 	ret->ft_load_flags = 0;
 	ret->ft_render_flags = 0;
 	ret->width = 0;
@@ -73,64 +73,36 @@ _vte_glyph_cache_new(void)
 	return ret;
 }
 
-void
-_vte_glyph_free(struct _vte_glyph *glyph)
-{
-	g_free(glyph);
-}
-
-static gboolean
-free_tree_value(gpointer key, gpointer value, gpointer data)
-{
-	if (GPOINTER_TO_INT(value) != INVALID_GLYPH) {
-		_vte_glyph_free(value);
-	}
-	return FALSE;
-}
 
 void
 _vte_glyph_cache_free(struct _vte_glyph_cache *cache)
 {
-	GList *iter;
-	int i;
+	guint i;
 
 	g_return_if_fail(cache != NULL);
 
 	/* Destroy the patterns. */
 	if (cache->patterns != NULL) {
 		for (i = 0; i < cache->patterns->len; i++) {
-			FcPatternDestroy(g_array_index(cache->patterns,
-						       FcPattern*,
-						       i));
+			FcPatternDestroy(g_ptr_array_index(cache->patterns, i));
 		}
-		g_array_free(cache->patterns, TRUE);
-		cache->patterns = NULL;
+		g_ptr_array_free(cache->patterns, TRUE);
 	}
 
 	/* Close all faces. */
-	for (iter = cache->faces; iter != NULL; iter = g_list_next(iter)) {
-		FT_Done_Face((FT_Face) iter->data);
-		iter->data = NULL;
-	}
-	cache->faces = NULL;
+	g_list_foreach(cache->faces, (GFunc)FT_Done_Face, NULL);
+	g_list_free(cache->faces);
 
 	/* Free the glyph tree. */
-	g_tree_foreach(cache->cache, free_tree_value, NULL);
-	cache->cache = NULL;
+	g_hash_table_destroy(cache->cache);
 
 	/* Close the FT library. */
 	if (cache->ft_library) {
 		FT_Done_FreeType(cache->ft_library);
-		cache->ft_library = NULL;
 	}
 
 	/* Free the cache. */
-	cache->ft_load_flags = 0;
-	cache->ft_render_flags = 0;
-	cache->width = 0;
-	cache->height = 0;
-	cache->ascent = 0;
-	g_free(cache);
+	g_slice_free(struct _vte_glyph_cache, cache);
 }
 
 void
@@ -143,31 +115,32 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 				      gpointer defaults_data)
 {
 	FcChar8 *facefile;
-	int i, j, error, count, width, faceindex;
+	int b, error, count, width, faceindex;
+	guint i,j;
 	double dpi, size;
 	GList *iter;
 	FcPattern *pattern;
-	GArray *patterns;
-	FT_Face face;
+	GPtrArray *patterns;
+	FT_Face face, prev_face;
 	gunichar double_wide_characters[] = {VTE_DRAW_DOUBLE_WIDE_CHARACTERS};
 
 	g_return_if_fail(cache != NULL);
 	g_return_if_fail(fontdesc != NULL);
 
 	/* Convert the font description to a sorted set of patterns. */
-	patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
+	patterns = g_ptr_array_new();
 	if (!_vte_fc_patterns_from_pango_font_desc(widget, fontdesc,
 						   antialias,
 						   patterns,
 						   defaults_cb,
 						   defaults_data)) {
-		g_array_free(patterns, TRUE);
+		g_ptr_array_free(patterns, TRUE);
 		g_assert_not_reached();
 	}
 
 	/* Set the pattern list. */
 	if (cache->patterns != NULL) {
-		g_array_free(cache->patterns, TRUE);
+		g_ptr_array_free(cache->patterns, TRUE);
 	}
 	cache->patterns = patterns;
 
@@ -180,9 +153,8 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 	cache->faces = NULL;
 
 	/* Clear the glyph tree. */
-	g_tree_foreach(cache->cache, free_tree_value, NULL);
-	g_tree_destroy(cache->cache);
-	cache->cache = g_tree_new(_vte_direct_compare);
+	g_hash_table_destroy(cache->cache);
+	cache->cache = g_hash_table_new_full(NULL, NULL, NULL, _vte_cached_glyph_free);
 
 	/* Clear the load and render flags. */
 	cache->ft_load_flags = 0;
@@ -190,7 +162,7 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 
 	/* Open the all of the faces to which the patterns resolve. */
 	for (i = 0; i < cache->patterns->len; i++) {
-		pattern = g_array_index(cache->patterns, FcPattern*, i);
+		pattern = g_ptr_array_index(cache->patterns, i);
 		j = 0;
 		while (FcPatternGetString(pattern, FC_FILE, j,
 					  &facefile) == FcResultMatch) {
@@ -230,19 +202,19 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 	/* Pull out other settings. */
 	cache->ft_load_flags = 0;
 	cache->ft_render_flags = 0;
-	i = 0;
-	pattern = g_array_index(cache->patterns, FcPattern*, 0);
+	b = 0;
+	pattern = g_ptr_array_index(cache->patterns, 0);
 	/* Read and set the "use the autohinter", er, hint. */
 #if defined(FC_AUTOHINT) && defined(FT_LOAD_FORCE_AUTOHINT)
-	if (FcPatternGetBool(pattern, FC_AUTOHINT, 0, &i) == FcResultMatch) {
-		if (i != 0) {
+	if (FcPatternGetBool(pattern, FC_AUTOHINT, 0, &b) == FcResultMatch) {
+		if (b != 0) {
 			cache->ft_load_flags |= FT_LOAD_FORCE_AUTOHINT;
 		}
 	}
 #endif
 	/* Read and set the "use antialiasing" hint. */
-	if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0, &i) == FcResultMatch) {
-		if (i == 0) {
+	if (FcPatternGetBool(pattern, FC_ANTIALIAS, 0, &b) == FcResultMatch) {
+		if (b == 0) {
 			cache->ft_load_flags |= FT_LOAD_MONOCHROME;
 #if HAVE_DECL_FT_RENDER_MODE_MONO
 			cache->ft_render_flags = FT_RENDER_MODE_MONO;
@@ -253,14 +225,14 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 		}
 	}
 	/* Read and set the "hinting" hint. */
-	if (FcPatternGetBool(pattern, FC_HINTING, 0, &i) == FcResultMatch) {
-		if (i == 0) {
+	if (FcPatternGetBool(pattern, FC_HINTING, 0, &b) == FcResultMatch) {
+		if (b == 0) {
 			cache->ft_load_flags |= FT_LOAD_NO_HINTING;
 		} else {
 #if defined(FC_AUTOHINT) && defined(FT_LOAD_FORCE_AUTOHINT)
 			if (FcPatternGetBool(pattern, FC_AUTOHINT, 0,
-					     &i) == FcResultMatch) {
-				if (i != 0) {
+					     &b) == FcResultMatch) {
+				if (b != 0) {
 					cache->ft_render_flags |=
 						FT_LOAD_FORCE_AUTOHINT;
 				}
@@ -268,8 +240,8 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 #endif
 #ifdef FC_HINT_STYLE
 			if (FcPatternGetInteger(pattern, FC_HINT_STYLE, 0,
-						&i) == FcResultMatch) {
-				switch (i) {
+						&b) == FcResultMatch) {
+				switch (b) {
 #if HAVE_DECL_FT_LOAD_NO_HINTING
 				case FC_HINT_NONE:
 					cache->ft_load_flags |=
@@ -349,7 +321,9 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 		cache->height = 1;
 		cache->ascent = 1;
 	}
-	width = 0;
+	/* detect if font measured above has the fixed-width property */
+	width = count = 0;
+	prev_face = NULL;
 	for (i = 0; i < G_N_ELEMENTS(double_wide_characters); i++) {
 		face = _vte_glyph_cache_face_for_char(cache,
 						      double_wide_characters[i]);
@@ -364,13 +338,27 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 						cache->ft_render_flags);
 		}
 		if (error == 0) {
+			if (count && prev_face != face) {
+				width /= 64 * count;
+				if (cache->width >= width - 1 &&
+						cache->width <= width + 1) {
+					/* add 1 to round up when dividing by 2 */
+					cache->width = (cache->width + 1) / 2;
+					break;
+				}
+				count = width =0;
+			}
 			width += face->glyph->metrics.horiAdvance;
 			count++;
+			prev_face = face;
 		}
 	}
 	if (count > 0) {
-		if (cache->width == width / 64 / count) {
-			cache->width /= 2;
+		width /= 64 * count;
+		if (cache->width >= width - 1 &&
+				cache->width <= width + 1) {
+			/* add 1 to round up when dividing by 2 */
+			cache->width = (cache->width + 1) / 2;
 		}
 	}
 }
@@ -393,8 +381,8 @@ _vte_glyph_cache_has_char(struct _vte_glyph_cache *cache, gunichar c)
 	GList *iter;
 	gpointer p;
 
-	if ((p = g_tree_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
-		if (GPOINTER_TO_INT(p) == INVALID_GLYPH) {
+	if ((p = g_hash_table_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
+		if (p == INVALID_GLYPH) {
 			return FALSE;
 		}
 	}
@@ -409,14 +397,10 @@ _vte_glyph_cache_has_char(struct _vte_glyph_cache *cache, gunichar c)
 }
 
 static gunichar
-_vte_glyph_remap_char(struct _vte_glyph_cache *cache, gunichar origc)
+_vte_glyph_remap_char(gunichar origc)
 {
 	gunichar newc;
 
-	if (_vte_glyph_cache_has_char(cache, origc)) {
-		return origc;
-	}
-
 	switch (origc) {
 	case 0:			/* NUL */
 	case 0x00A0:		/* NO-BREAK SPACE */
@@ -435,11 +419,7 @@ _vte_glyph_remap_char(struct _vte_glyph_cache *cache, gunichar origc)
 		break;
 	}
 
-	if (_vte_glyph_cache_has_char(cache, newc)) {
-		return newc;
-	} else {
-		return origc;
-	}
+	return newc;
 }
 
 #define DEFAULT_BYTES_PER_PIXEL 3
@@ -497,8 +477,8 @@ _vte_glyph_get_uncached(struct _vte_glyph_cache *cache, gunichar c)
 
 	/* Bail if we weren't able to load the glyph. */
 	if (face == NULL) {
-		g_tree_insert(cache->cache, GINT_TO_POINTER(c),
-			      GINT_TO_POINTER(INVALID_GLYPH));
+		g_hash_table_insert(cache->cache, GINT_TO_POINTER(c),
+			      INVALID_GLYPH);
 		return NULL;
 	}
 
@@ -513,9 +493,6 @@ _vte_glyph_get_uncached(struct _vte_glyph_cache *cache, gunichar c)
 			  face->glyph->bitmap_top, 0);
 	glyph->bytes_per_pixel = DEFAULT_BYTES_PER_PIXEL;
 
-	memset(glyph->bytes, 0,
-	       glyph->width * glyph->height * DEFAULT_BYTES_PER_PIXEL);
-
 	for (y = 0; y < face->glyph->bitmap.rows; y++)
 	for (x = 0; x < face->glyph->bitmap.width; x++) {
 		ooffset = (y * glyph->width + x) * DEFAULT_BYTES_PER_PIXEL;
@@ -608,8 +585,8 @@ _vte_glyph_get(struct _vte_glyph_cache *cache, gunichar c)
 	g_return_val_if_fail(cache != NULL, NULL);
 
 	/* See if we already have a glyph for this character. */
-	if ((p = g_tree_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
-		if (GPOINTER_TO_INT(p) == INVALID_GLYPH) {
+	if ((p = g_hash_table_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
+		if (p == INVALID_GLYPH) {
 			return NULL;
 		} else {
 			return p;
@@ -621,13 +598,13 @@ _vte_glyph_get(struct _vte_glyph_cache *cache, gunichar c)
 
 	/* Bail if we weren't able to load the glyph. */
 	if (glyph == NULL) {
-		g_tree_insert(cache->cache, GINT_TO_POINTER(c),
-			      GINT_TO_POINTER(INVALID_GLYPH));
+		g_hash_table_insert(cache->cache, GINT_TO_POINTER(c),
+			      INVALID_GLYPH);
 		return NULL;
 	}
 
 	/* Cache it. */
-	g_tree_insert(cache->cache, GINT_TO_POINTER(c), glyph);
+	g_hash_table_insert(cache->cache, GINT_TO_POINTER(c), glyph);
 
 	return glyph;
 }
@@ -640,7 +617,7 @@ _vte_glyph_draw(struct _vte_glyph_cache *cache,
 		struct _vte_rgb_buffer *buffer)
 {
 	const struct _vte_glyph *glyph;
-	gint col, row, ioffset, ooffset, icol, ocol, ecol;
+	gint col, row, erow, ioffset, ooffset, icol, ocol, ecol;
 	gint strikethrough, underline, underline2;
 	gint32 r, g, b, ar, ag, ab;
 	guchar *pixels;
@@ -648,9 +625,14 @@ _vte_glyph_draw(struct _vte_glyph_cache *cache,
 	if (cache == NULL) {
 		return;
 	}
-	glyph = _vte_glyph_get(cache, _vte_glyph_remap_char(cache, c));
+	glyph = _vte_glyph_get(cache, c);
 	if (glyph == NULL) {
-		return;
+		gunichar cc = _vte_glyph_remap_char(c);
+		if(cc != c)
+			glyph = _vte_glyph_get(cache, cc);
+		if (glyph == NULL) {
+			return;
+		}
 	}
 
 	if (x > buffer->width) {
@@ -680,21 +662,18 @@ _vte_glyph_draw(struct _vte_glyph_cache *cache,
 
 _vte_glyph_draw_loop:
 
+	erow = MIN(cache->height, glyph->skip + glyph->height);
+	erow = MIN(erow, buffer->height - y);
 	for (row = glyph->skip;
-	     row < MIN(cache->height, glyph->skip + glyph->height);
+	     row < erow;
 	     row++) {
-		if (row + y >= buffer->height) {
-			break;
-		}
 		ooffset = (y + row) * buffer->stride +
 			  ((x + ocol) * 3);
 		ioffset = (((row - glyph->skip) * glyph->width) + icol) *
 			  DEFAULT_BYTES_PER_PIXEL;
 		ecol = MIN(cache->width * columns, glyph->width);
+		ecol = MIN(ecol, buffer->width - x);
 		for (col = 0; col < ecol; col++) {
-			if (col + x >= buffer->width) {
-				break;
-			}
 			ar = glyph->bytes[ioffset + 0];
 			ag = glyph->bytes[ioffset + 1];
 			ab = glyph->bytes[ioffset + 2];
