@@ -139,6 +139,16 @@ _vte_xft_font_open (GtkWidget *widget, const PangoFontDescription *fontdesc,
 {
 	struct _vte_xft_font *font, *old;
 	GPtrArray *patterns;
+        Display *dpy;
+        int event_base, error_base;
+
+        /* Don't enable anti-aliasing unless the XRender extension is available */
+        dpy = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (widget));
+        if (antialias == VTE_ANTI_ALIAS_USE_DEFAULT &&
+            (!XRenderQueryExtension (dpy, &event_base, &error_base) ||
+             !XRenderFindVisualFormat (dpy, DefaultVisual (dpy, DefaultScreen (dpy))))) {
+                antialias = VTE_ANTI_ALIAS_FORCE_DISABLE;
+        }
 
 	patterns = g_ptr_array_new ();
 	if (!_vte_fc_patterns_from_pango_font_desc (widget, fontdesc, antialias,
@@ -149,7 +159,7 @@ _vte_xft_font_open (GtkWidget *widget, const PangoFontDescription *fontdesc,
 
 	font = g_slice_new (struct _vte_xft_font);
 	font->ref = 1;
-	font->display = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (widget));
+	font->display = dpy;
 	font->patterns = patterns;
 	font->last_pattern = 0;
 	font->have_metrics = FALSE;
@@ -262,7 +272,9 @@ _vte_xft_open_font_for_char (struct _vte_xft_font *font, gunichar c, GPtrArray *
 	/* No match? */
 	_vte_tree_insert (font->fontmap,
 			p, GINT_TO_POINTER (-FONT_INDEX_FUDGE));
-	g_warning (_ ("Can not find appropiate font for character U+%04x.\n"), c);
+	_vte_debug_print (VTE_DEBUG_MISC,
+			"Can not find appropriate font for character U+%04x.\n",
+			c);
 	return NULL;
 }
 static inline XftFont *
@@ -379,6 +391,9 @@ _vte_xft_destroy (struct _vte_draw *draw)
 		_vte_xft_unlock_fonts (data);
 		_vte_xft_font_close (data->font);
 	}
+	if (data->pixmap != NULL) {
+		g_object_unref (data->pixmap);
+	}
 	if (data->draw != NULL) {
 		XftDrawDestroy (data->draw);
 	}
@@ -478,25 +493,30 @@ _vte_xft_set_background_image (struct _vte_draw *draw,
 			      double saturation)
 {
 	struct _vte_xft_data *data;
-	GdkPixmap *pixmap;
-	GdkScreen *screen;
+	GdkPixmap *pixmap = NULL;
 
 	data = (struct _vte_xft_data*) draw->impl_data;
 
-	screen = gtk_widget_get_screen (draw->widget);
-
+	draw->requires_clear = data->opacity != 0xffff;
 	data->xpixmap = -1;
 	data->pixmapw = data->pixmaph = 0;
-	pixmap = vte_bg_get_pixmap (vte_bg_get_for_screen (screen), type,
-				   pixbuf, file, color, saturation,
-				   _vte_draw_get_colormap (draw, TRUE));
+
 	if (data->pixmap != NULL) {
 		g_object_unref (data->pixmap);
 	}
-	draw->requires_clear = data->opacity != 0xffff;
-	data->pixmap = NULL;
+
+	if (type != VTE_BG_SOURCE_NONE) {
+		GdkScreen *screen;
+
+		screen = gtk_widget_get_screen (draw->widget);
+		pixmap = vte_bg_get_pixmap (vte_bg_get_for_screen (screen),
+				           type, pixbuf, file,
+					   color, saturation,
+					   _vte_draw_get_colormap (draw, TRUE));
+	}
+
+	data->pixmap = pixmap;
 	if (pixmap != NULL) {
-		data->pixmap = pixmap;
 		data->xpixmap = gdk_x11_drawable_get_xid (pixmap);
 		gdk_drawable_get_size (pixmap, &data->pixmapw, &data->pixmaph);
 		draw->requires_clear |=
@@ -995,7 +1015,7 @@ _vte_xft_set_scroll (struct _vte_draw *draw, gint x, gint y)
 }
 
 const struct _vte_draw_impl _vte_draw_xft = {
-	"Xft",
+	"xft",
 	_vte_xft_check,
 	_vte_xft_create,
 	_vte_xft_destroy,
